""" Infrastructure for loopy transformations.
These are registered to list them in interactive mode
"""

_loopy_trafo_registry = {}


def get_loopy_transformations():
    return _loopy_trafo_registry


class LoopyTransformationWrapper(object):
    def __init__(self, f, name=None, description=""):
        self.func = f
        self.name = name
        self.description = description

        assert name
        assert name not in _loopy_trafo_registry

        _loopy_trafo_registry[name] = self

    def __call__(self, kernel):
        return self.func(kernel)


def loopy_transformation(_positional_arg=None, **kwargs):
    assert not _positional_arg
    return lambda f: LoopyTransformationWrapper(f, **kwargs)


# Just for debugging purposes we add an identity transformation here.
@loopy_transformation(name="identity", description='''Does not change the kernel. Proof of concept implementation''')
def _identity(kernel):
    return kernel
